#!/usr/bin/env python3
from typing import List, Optional
import argparse
import sys

from path import Path

from .checker import Checker


def add_word(word: str, *, lang: str) -> None:
    checker = Checker(lang=lang)
    checker.add(word)
    print("Word added to", checker.pwl_path)


def remove_word(word: str, *, lang: str) -> None:
    checker = Checker(lang=lang)
    checker.remove(word)
    print("Word removed from", checker.pwl_path)


def check(path: Path, *, lang: str, filetype: Optional[str] = None) -> bool:
    checker = Checker(lang=lang)
    errors = checker.check(path, filetype=filetype)
    ok = True
    for error in errors:
        ok = False
        print(f"{path}:{error.line}:{error.offset}: error: {error.word}")
    return ok


def kak_menu_from_replacements(replacements: List[str]) -> str:
    menu = ""
    for entry in replacements:
        # Note: %{...} is kakoune way of grouping stuff that may - or not
        # contains quotes, which prevents us from using `%`, `.format()` or
        # f-strings :P
        menu_entry = "%{ENTRY} %{execute-keys -itersel %{cENTRY<esc>be}}"
        menu_entry = menu_entry.replace("ENTRY", entry)
        menu += " " + menu_entry
    return "menu " + menu


def replace(word: str, *, lang: str, kak_output: bool) -> None:
    checker = Checker(lang=lang)
    replacements = checker.replace(word)
    if kak_output:
        menu = kak_menu_from_replacements(replacements)
        print(menu)
    else:
        print(" ".join(replacements))


def main(argv: Optional[List[str]] = None) -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("--lang", default="en_US")

    subparsers = parser.add_subparsers(title="commands", dest="command")

    check_parser = subparsers.add_parser("check")
    check_parser.add_argument("path", type=Path)
    check_parser.add_argument(
        "--filetype", help="file type, as set by kakoune in $kak_opt_filetype"
    )

    add_parser = subparsers.add_parser("add")
    add_parser.add_argument("word")

    remove_parser = subparsers.add_parser("remove")
    remove_parser.add_argument("word")

    replace_parser = subparsers.add_parser("replace")
    replace_parser.add_argument(
        "--kakoune", action="store_true", help="Output kak-script compatible output"
    )
    replace_parser.add_argument("word")

    args = parser.parse_args(args=argv)
    lang = args.lang

    if args.command == "add":
        word = args.word
        add_word(word, lang=lang)
    elif args.command == "remove":
        word = args.word
        remove_word(word, lang=lang)
    elif args.command == "check":
        path = args.path
        ok = check(path, lang=lang, filetype=args.filetype)
        if not ok:
            sys.exit(1)
    elif args.command == "replace":
        word = args.word
        kakoune = args.kakoune
        replace(word, lang=lang, kak_output=kakoune)
    else:
        parser.print_help()
        sys.exit(1)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kak_spell']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=19.3.0,<20.0.0',
 'path>=13.1.0,<14.0.0',
 'pyenchant==3.0.0rc3',
 'pyxdg>=0.26,<0.27']

entry_points = \
{'console_scripts': ['kak-spell = kak_spell.cli:main']}

setup_kwargs = {
    'name': 'kak-spell',
    'version': '0.1.0',
    'description': 'PyEnchant wrapper for Kakoune',
    'long_description': '# kak-spell\n\nPyEnchant wrapper for Kakoune.\n\n## Installation\n\n\n1. Install the Enchant library, and the required dictionaries. See [PyEnchant documentation](https://pyenchant.github.io/pyenchant/install.html) for details.\n\n2. Install the `kak-spell` script, for instance with [pipx](https://pipxproject.github.io/pipx/):\n\n```\npipx install kak-spell\n```\n\n3. Install [plug.kak](https://github.com/andreyorst/plug.kak) and add the following lines in your `kakrc`:\n\n```kak\nplug "dmerejkowsky/kak-spell"\n```\n\n4. (optional): declare a user mode and some mappings:\n\n```kak\ndeclare-user-mode kak-spell\nmap global user s \': enter-user-mode kak-spell<ret>\' -docstring \'enter spell user mode\'\nmap global kak-spell e \': kak-spell %opt{spell_lang}<ret>\' -docstring "enable spell checking"\nmap global kak-spell c \': kak-spell-clear<ret>\' -docstring \'clear spelling highlighters\'\nmap global kak-spell n \': kak-spell-next<ret>\' -docstring \'go to next spell error\'\nmap global kak-spell r \': kak-spell-replace<ret>\' -docstring \'suggest a list of replacements\'\nmap global kak-spell a \': kak-spell-add<ret>\' -docstring \'add the selection to the user dict\'\nmap global kak-spell x \': kak-spell-remove<ret>\' -docstring \'remove the selection from the user dict\'\n```\n\n\n\n## Discuss\n\nYou can discuss features of this plugin on [discuss.kakoune.com](https://discuss.kakoune.com/t/alternate-implementation-for-spell-checker/781).\n\nI\'d like to thank all the people who contributed code and ideas to make usage of this plugin easier.\n',
    'author': 'Dimitri Merejkowsky',
    'author_email': 'd.merej@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dmerejkowsky/kak-spell',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
